/*--------------------------------------------------------------------------*/
// Sample Application													
//																			
// Chapter 2																
//																			
// Learn DirectX in 24 Hours												
// by Robert Dunlop															
//																			
// Copyright (C) 1999														
/*--------------------------------------------------------------------------*/

#ifndef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN
#endif

//------ Include Files ------//

#include "stdafx.h"
#include "resource.h"
#define INITGUID
#include <ddraw.h>

//------ Image Loading and Initialization Flags ------//

BOOL bInit=FALSE;

//------ Global Interface Pointers ------//

LPDIRECTDRAW7               lpDD=NULL;          // DirectDraw object
LPDIRECTDRAWSURFACE7        lpDDSPrimary=NULL;  // DirectDraw primary surface
LPDIRECTDRAWCLIPPER			lpClip=NULL;		// Directray Clipper Object
LPDIRECTDRAWSURFACE7		lpBmp=NULL;			// Bitmap surface

//------ Window Class Information ------//

static char                 szClass[] = "XmplHr2Class";
static char                 szCaption[] = "Example - Hour 2";

//------ Error Return String ------//

const char *ErrStr;			

//------ Error Messages ------//

const char Err_Reg_Class[]			= "Error Registering Window Class";
const char Err_Create_Win[]			= "Error Creating Window";
const char Err_DirectDrawCreate[]	= "DirectDrawCreate FAILED";
const char Err_Coop[]				= "SetCooperativeLevel FAILED";
const char Err_CreateClip[]			= "CreateClip FAILED";
const char Err_CreateSurf[]			= "CreateSurface FAILED";
const char Err_LoadBMP[]			= "Error Loading Image";

//------ Cleanup Function to Release Objects ------//

#define SafeRelease(x) if (x) { x->Release(); x=NULL; }

void Cleanup(void)
{
	// release the interfaces

	SafeRelease(lpBmp);
	SafeRelease(lpDDSPrimary);
	SafeRelease(lpClip);
	SafeRelease(lpDD);

	// display error if one thrown

	if (ErrStr)
		MessageBox(NULL, ErrStr, szCaption, MB_OK);
}

//------ Function to Load a Bitmap into a DirectDraw Surface ------//

LPDIRECTDRAWSURFACE7 bitmap_surface(LPCTSTR file_name)
{
	HDC hdc;
	HBITMAP bit;
	LPDIRECTDRAWSURFACE7 surf;

	// load the interface bitmap

	bit=(HBITMAP) LoadImage(NULL,file_name,IMAGE_BITMAP,0,0,
								LR_DEFAULTSIZE|LR_LOADFROMFILE);
	if (!bit) 

		// failed to load, return failure to caller

		return NULL;

	// get bitmap dimensions

	BITMAP bitmap;
    GetObject( bit, sizeof(BITMAP), &bitmap );
	int surf_width=bitmap.bmWidth;
	int surf_height=bitmap.bmHeight;

	// create surface

	HRESULT ddrval;
	DDSURFACEDESC2 ddsd;
	ZeroMemory(&ddsd,sizeof(ddsd));
	ddsd.dwSize = sizeof(DDSURFACEDESC2);
	ddsd.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT ;
	ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN|DDSCAPS_SYSTEMMEMORY; 
	ddsd.dwWidth = surf_width;
	ddsd.dwHeight = surf_height; 

	// attempt to create surface

	ddrval=lpDD->CreateSurface(&ddsd,&surf,NULL);

	// created ok?

	if (ddrval!=DD_OK) {

		// no, release the bitmap and return failure to caller

		DeleteObject(bit);
		return NULL;

	} else {

		// yes, get a DC for the surface

		surf->GetDC(&hdc);

		// generate a compatible DC

		HDC bit_dc=CreateCompatibleDC(hdc);

		// blit the interface to the surface

		SelectObject(bit_dc,bit);
		BitBlt(hdc,0,0,surf_width,surf_height,bit_dc,0,0,SRCCOPY);

		// release the DCs

		surf->ReleaseDC(hdc);
		DeleteDC(bit_dc);
	}

	// clear bitmap 

	DeleteObject(bit);

	// return pointer to caller

	return surf;
}

//------ Draw the Bitmap to the Primary Surface ------//

void DrawImage()
{
	// return if not ready to draw at this time

    if (!lpBmp||!bInit)
        return;

	// draw the image full screen

	lpDDSPrimary->Blt(NULL,lpBmp,NULL,DDBLT_WAIT,NULL);
}

//------ Windows Message Handler ------//

LRESULT CALLBACK WindowProc(HWND hWnd, unsigned uMsg, WPARAM wParam, LPARAM lParam)
{    
    switch (uMsg)
    {
        case WM_COMMAND:
            switch (LOWORD(wParam))
            {
                case IDM_EXIT:

                    DestroyWindow(hWnd);
                    break;
            }
            break;

        case WM_PAINT:
			PAINTSTRUCT ps;

            BeginPaint(hWnd, &ps);
			DrawImage();
            EndPaint(hWnd, &ps);
            break;

        case WM_DESTROY:
            Cleanup();
            PostQuitMessage(0);
            break;

        default:
            return DefWindowProc(hWnd, uMsg, wParam, lParam);
    }
    return 0L;
}

//------ Function to Initialize DirectDraw and the Application ------//

static BOOL Init(HINSTANCE hInstance, int nCmdShow)
{
    WNDCLASS                    wc;
    HRESULT                     hRet;
    DDSURFACEDESC2              ddsd;

    // Set up and register window class

    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = (WNDPROC) WindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = sizeof(DWORD);
    wc.hInstance = hInstance;
    wc.hIcon = NULL;
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH) GetStockObject(BLACK_BRUSH);
    wc.lpszMenuName = MAKEINTRESOURCE(IDR_MENU);
    wc.lpszClassName = szClass;
    if (!RegisterClass(&wc)) {
		ErrStr=Err_Reg_Class;
        return FALSE;
	}

    // Get dimensions of display

    int ScreenWidth = GetSystemMetrics(SM_CXSCREEN);
    int ScreenHeight = GetSystemMetrics(SM_CYSCREEN);

    // Create a window and display

	HWND hWnd;

    hWnd = CreateWindow(szClass,							// class
                        szCaption,							// caption
						WS_VISIBLE|WS_POPUP,				// style 
						0,									// left
						0,									// top
						ScreenWidth,						// width
						ScreenHeight,						// height
                        NULL,								// parent window
                        NULL,								// menu 
                        hInstance,							// instance
                        NULL);								// parms
    if (!hWnd) {
		ErrStr=Err_Create_Win;
        return FALSE;
	}
    ShowWindow(hWnd, nCmdShow);
    UpdateWindow(hWnd);

    // Create the main DirectDraw object

    hRet = DirectDrawCreateEx(NULL, (VOID **) &lpDD, IID_IDirectDraw7,NULL);
    if (hRet != DD_OK) {
		ErrStr=Err_DirectDrawCreate;
		return FALSE;
	}

    // Set our cooperative level

    hRet = lpDD->SetCooperativeLevel(hWnd, DDSCL_NORMAL);
    if (hRet != DD_OK) {
		ErrStr=Err_Coop;
		return FALSE;
	}

	// create the clipper

	hRet=lpDD->CreateClipper(NULL,&lpClip,NULL);
    if (hRet != DD_OK) {
		ErrStr=Err_CreateClip;
		return FALSE;
	}

	// set the window for the clipper

	lpClip->SetHWnd(0,hWnd);

    // Create the primary surface

    ZeroMemory(&ddsd, sizeof(ddsd));
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_CAPS;
    ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;
    hRet = lpDD->CreateSurface(&ddsd, &lpDDSPrimary, NULL);
    if (hRet != DD_OK) {
		ErrStr=Err_CreateSurf;
		return FALSE;
	}

	// Set the Clipper for the Primary Surface

	lpDDSPrimary->SetClipper(lpClip);

	// flag initialization as completed

	bInit=TRUE;

	// load the default bitmap

	lpBmp=bitmap_surface("vista.bmp");
	if (!lpBmp) {
		ErrStr=Err_LoadBMP;
		return FALSE;
	}

	// display the image

	DrawImage();

    return TRUE;
}

//------ Application Loop ------//

int PASCAL WinMain( HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPSTR lpCmdLine,
					int nCmdShow)
{
    MSG msg;		// windows message structure

	// initialize the application, exit on failure

    if (!Init(hInstance, nCmdShow)) {
		Cleanup();
        return FALSE;
	}

	// handle the message loop till we exit

    while (GetMessage(&msg, NULL, NULL, NULL))
    {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }

	// exit returning final message

    return (msg.wParam);
}



